
From: Karsten Wiese <annabellesgarden@yahoo.de>

IRQ_PER_CPU is not used by all architectures.  This patch introduces the
macros ARCH_HAS_IRQ_PER_CPU and CHECK_IRQ_PER_CPU() to avoid the generation
of dead code in __do_IRQ().

ARCH_HAS_IRQ_PER_CPU is defined by architectures using IRQ_PER_CPU in their
include/asm_ARCH/irq.h file.

Through grepping the tree I found the following architectures currently use
IRQ_PER_CPU:

        cris, ia64, ppc, ppc64 and parisc. 

Signed-off-by: Karsten Wiese <annabellesgarden@yahoo.de>
Acked-by: Ingo Molnar <mingo@elte.hu>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 include/asm-cris/irq.h   |    5 +++++
 include/asm-ia64/irq.h   |    5 +++++
 include/asm-parisc/irq.h |    5 +++++
 include/asm-ppc/irq.h    |    5 +++++
 include/asm-ppc64/irq.h  |    5 +++++
 include/linux/irq.h      |    7 ++++++-
 kernel/irq/handle.c      |    2 +-
 7 files changed, 32 insertions(+), 2 deletions(-)

diff -puN include/asm-cris/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq include/asm-cris/irq.h
--- devel/include/asm-cris/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq	2005-08-08 22:35:18.000000000 -0700
+++ devel-akpm/include/asm-cris/irq.h	2005-08-08 22:35:18.000000000 -0700
@@ -1,6 +1,11 @@
 #ifndef _ASM_IRQ_H
 #define _ASM_IRQ_H
 
+/*
+ * IRQ line status macro IRQ_PER_CPU is used
+ */
+#define ARCH_HAS_IRQ_PER_CPU
+
 #include <asm/arch/irq.h>
 
 extern __inline__ int irq_canonicalize(int irq)
diff -puN include/asm-ia64/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq include/asm-ia64/irq.h
--- devel/include/asm-ia64/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq	2005-08-08 22:35:18.000000000 -0700
+++ devel-akpm/include/asm-ia64/irq.h	2005-08-08 22:35:18.000000000 -0700
@@ -14,6 +14,11 @@
 #define NR_IRQS		256
 #define NR_IRQ_VECTORS	NR_IRQS
 
+/*
+ * IRQ line status macro IRQ_PER_CPU is used
+ */
+#define ARCH_HAS_IRQ_PER_CPU
+
 static __inline__ int
 irq_canonicalize (int irq)
 {
diff -puN include/asm-parisc/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq include/asm-parisc/irq.h
--- devel/include/asm-parisc/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq	2005-08-08 22:35:18.000000000 -0700
+++ devel-akpm/include/asm-parisc/irq.h	2005-08-08 22:35:18.000000000 -0700
@@ -26,6 +26,11 @@
 
 #define NR_IRQS		(CPU_IRQ_MAX + 1)
 
+/*
+ * IRQ line status macro IRQ_PER_CPU is used
+ */
+#define ARCH_HAS_IRQ_PER_CPU
+
 static __inline__ int irq_canonicalize(int irq)
 {
 	return (irq == 2) ? 9 : irq;
diff -puN include/asm-ppc64/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq include/asm-ppc64/irq.h
--- devel/include/asm-ppc64/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq	2005-08-08 22:35:18.000000000 -0700
+++ devel-akpm/include/asm-ppc64/irq.h	2005-08-08 22:35:18.000000000 -0700
@@ -33,6 +33,11 @@
 #define IRQ_POLARITY_POSITIVE	0x2	/* high level or low->high edge */
 #define IRQ_POLARITY_NEGATIVE	0x0	/* low level or high->low edge */
 
+/*
+ * IRQ line status macro IRQ_PER_CPU is used
+ */
+#define ARCH_HAS_IRQ_PER_CPU
+
 #define get_irq_desc(irq) (&irq_desc[(irq)])
 
 /* Define a way to iterate across irqs. */
diff -puN include/asm-ppc/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq include/asm-ppc/irq.h
--- devel/include/asm-ppc/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq	2005-08-08 22:35:18.000000000 -0700
+++ devel-akpm/include/asm-ppc/irq.h	2005-08-08 22:35:18.000000000 -0700
@@ -19,6 +19,11 @@
 #define IRQ_POLARITY_POSITIVE	0x2	/* high level or low->high edge */
 #define IRQ_POLARITY_NEGATIVE	0x0	/* low level or high->low edge */
 
+/*
+ * IRQ line status macro IRQ_PER_CPU is used
+ */
+#define ARCH_HAS_IRQ_PER_CPU
+
 #if defined(CONFIG_40x)
 #include <asm/ibm4xx.h>
 
diff -puN include/linux/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq include/linux/irq.h
--- devel/include/linux/irq.h~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq	2005-08-08 22:35:18.000000000 -0700
+++ devel-akpm/include/linux/irq.h	2005-08-08 22:35:18.000000000 -0700
@@ -32,7 +32,12 @@
 #define IRQ_WAITING	32	/* IRQ not yet seen - for autodetection */
 #define IRQ_LEVEL	64	/* IRQ level triggered */
 #define IRQ_MASKED	128	/* IRQ masked - shouldn't be seen again */
-#define IRQ_PER_CPU	256	/* IRQ is per CPU */
+#if defined(ARCH_HAS_IRQ_PER_CPU)
+# define IRQ_PER_CPU	256	/* IRQ is per CPU */
+# define CHECK_IRQ_PER_CPU(var) ((var) & IRQ_PER_CPU)
+#else
+# define CHECK_IRQ_PER_CPU(var) 0
+#endif
 
 /*
  * Interrupt controller descriptor. This is all we need
diff -puN kernel/irq/handle.c~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq kernel/irq/handle.c
--- devel/kernel/irq/handle.c~check_irq_per_cpu-to-avoid-dead-code-in-__do_irq	2005-08-08 22:35:18.000000000 -0700
+++ devel-akpm/kernel/irq/handle.c	2005-08-08 22:35:18.000000000 -0700
@@ -128,7 +128,7 @@ fastcall unsigned int __do_IRQ(unsigned 
 	unsigned int status;
 
 	kstat_this_cpu.irqs[irq]++;
-	if (desc->status & IRQ_PER_CPU) {
+	if (CHECK_IRQ_PER_CPU(desc->status)) {
 		irqreturn_t action_ret;
 
 		/*
_
